<?php
/**
 * Frontend helper functions used throught the theme.
 *
 * @package Highend
 * @since   3.5.0
 */

/**
 * Do not allow direct script access.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! function_exists( 'highend_get_focus_color' ) ) :

	/**
	 * Return theme focus color.
	 * 
	 * @since 3.5.0
	 */
	function highend_get_focus_color() {

		return apply_filters( 'highend_focus_color', get_theme_mod( 'hb_focus_color_setting', '#1dc6df' ) );
	}
endif;

if ( ! function_exists( 'highend_get_user_social_links' ) ) :
	/**
	 * Retrieve user social network links from profile.
	 * 
	 * @since 3.5.0
	 */
	function highend_get_user_social_links( $user_id ) {

		$networks = highend_get_social_networks_array();
		$return   = array();
		
		foreach ( $networks as $network_id => $network_name ) {

			$return[ $network_id ] = array(
				'soc_name' => $network_name,
				'soc_link' => get_user_meta( $user_id, $network_id, true ),
			);
		}

		return $return;
	}
endif;

if ( ! function_exists( 'highend_get_image_dimensions' ) ) :
	/**
	 * Get image dimension based on orientation, ratio and width.
	 * 
	 * @since 3.5.0
	 */
	function highend_get_image_dimensions( $orientation = 'landscape', $ratio = 'ratio1', $width = 600 ) {

		$height = 0;

		switch ( $ratio ) {

			case 'ratio1':

				if ( 'portrait' === $orientation)
					$height = (int) (( $width / 9 ) * 16);
				else
					$height = (int) (( $width / 16 ) * 9);
				break;

			case 'ratio2':

				if ( 'portrait' === $orientation)
					$height = (int) (( $width / 3 ) * 4);
				else
					$height = (int) (( $width / 4 ) * 3);
				break;

			case 'ratio3';
				$height = (int) ( $width );
			break;

			case 'ratio4':
				if ( 'portrait' === $orientation )
					$height = (int) (( $width / 2 ) * 3);
				else
					$height = (int) (( $width / 3 ) * 2);
				break;

			case 'ratio5':
				if ( 'portrait' === $orientation )
					$height = (int) (( $width ) * 3);
				else
					$height = (int) (( $width / 3 ));
				break;
		}

		return array( 'width' => $width, 'height' => $height );
	}
endif;

if ( ! function_exists( 'highend_pagination_standard' ) ) :
	/**
	 * Display standard WP pagination.
	 * 
	 * @since 3.5.0
	 */
	function highend_pagination_standard() {

		global $wp_query;

		$big = 99999999;
		echo '<div class="clear"></div>';
		echo '<div class="pagination">';

		echo paginate_links( array(
			'base'      => str_replace( $big, '%#%', esc_url( get_pagenum_link( $big ) ) ),
			'format'    => '?paged=%#%',
			'total'     => $wp_query->max_num_pages,
			'current'   => max( 1, get_query_var('paged') ),
			'show_all'  => false,
			'end_size'  => 2,
			'mid_size'  => 1,
			'prev_next' => true,
			'prev_text' => '<i class="icon-angle-left"></i>',
			'next_text' => '<i class="icon-angle-right"></i>',
			'type'      => 'list'
		) );

		echo '</div>';
		echo '<div class="clear"></div>';
	}
endif;

if ( ! function_exists( 'highend_nl2p' ) ) :
	/**
	 * Convert line break (new line) to paragraphs.
	 *
	 * @since  3.5.0
	 * @param  string  $string      String to convert.
	 * @param  boolean $line_breaks Line breaks.
	 * @param  boolean $xml         XML.
	 */
	function highend_nl2p( $string, $line_breaks = false, $xml = true ) {

		// Remove existing HTML formatting to avoid double-wrapping things
		$string = str_replace( array( '<p>', '</p>', '<br>', '<br />' ), '', $string );
		
		// It is conceivable that people might still want single line-breaks without breaking into a new paragraph.
		if ( true === $line_breaks ) {
			return '<p>' . preg_replace( array( "/([\n]{2,})/i", "/([^>])\n([^<])/i"), array( "</p>\n<p>", '$1<br' . ( true === $xml ? ' /' : '' ) . '>$2' ), trim( $string ) ) . '</p>';
		} else {
			return '<p>' . preg_replace( array( "/([\n]{2,})/i", "/([\r\n]{3,})/i", "/([^>])\n([^<])/i" ), array( "</p>\n<p>", "</p>\n<p>", '$1<br' . ( true === $xml ? ' /' : '' ) . '>$2' ), trim( $string ) ) . '</p>'; 
		}
	}
endif;

if ( ! function_exists( 'highend_get_short_excerpt' ) ) :

	/**
	 * Get short excert by char count, but preserve whole words.
	 *
	 * @since  3.5.0
	 * @param  string  $text   Content.
	 * @param  boolean $length Max character count.
	 */
	function highend_get_short_excerpt( $text, $length ) {

		// Text is shorter than $length.
		if ( strlen( $text ) < $length ) {
			return $text;
		}

		$words  = explode( ' ', trim( $text ) );
		$return = null;

		if ( empty( $words ) ) {
			return;
		}

		foreach ( $words as $word ) {

			// Check if first work is longer than $length, if so, shorten and return it.
			if ( ( strlen( $word ) > $length ) && $return == null ) {
				return substr( $word, 0, $len ) . '...';
			}

			// Check length with next workd and if larger than $length, return.
			if ( ( strlen( $return ) + strlen( $word ) ) > $length ) {
				return $return . '...';
			}

			// Append word.
			$return .= ' ' . $word;
		}

		return $return;
	}
endif;

if ( ! function_exists( 'highend_map_json' ) ) :

	/**
	 * Get Google Map JSON.
	 *
	 * @since  3.5.0
	 */
	function highend_map_json() {

		$map = array();

		$map[1]['lat'] = hb_options( 'hb_map_1_latitude' );
		$map[1]['lng'] = hb_options( 'hb_map_1_longitude' );
		$map[1]['ibx'] = hb_options( 'hb_location_1_info' );

		$marker_count = apply_filters( 'highend_map_marker_number', 10 );

		for ( $i = 2; $i <= $marker_count; $i++ ){

			if ( hb_options( 'hb_enable_location_' . $i ) ) {
				$map[ $i ]['lat'] = hb_options( 'hb_map_' . $i . '_latitude' );
				$map[ $i ]['lng'] = hb_options( 'hb_map_' . $i . '_longitude' );
				$map[ $i ]['ibx'] = hb_options( 'hb_location_' . $i . '_info' );
			}
		}

		return $map;
	}
endif;

if ( ! function_exists( 'highend_get_post_format_icon' ) ) :

	/**
	 * Get post format icon
	 *
	 * @since  3.5.0
	 */
	function highend_get_post_format_icon( $format ) {

		$icons = array(
			'video'    => 'hb-moon-play-2',
			'status'   => 'hb-moon-pencil',
			'standard' => 'hb-moon-pencil',
			'gallery'  => 'hb-moon-image-3',
			'image'    => 'hb-moon-image-3',
			'audio'    => 'hb-mooon-music-2',
			'quote'    => 'hb-moon-quotes-right',
			'link'     => 'hb-moon-link-5',
		);

		$icon = isset( $icons[ $format ] ) ? $icons[ $format ] : 'hb-moon-file-3';

		return apply_filters( 'highend_post_format_icon', $icon, $format );	
	}
endif;

if ( ! function_exists( 'highend_is_seo_plugin_installed' ) ) :

	/**
	 * Check if SEO plugin is enabled.
	 *
	 * @since  3.5.0
	 */
	function highend_is_seo_plugin_installed() {
		return defined( 'WPSEO_VERSION' );
	}
endif;

if ( ! function_exists( 'highend_update_google_font_json' ) ) :

	/**
	 * Update Google fonts JSON.
	 *
	 * @since  3.5.0
	 */
	function highend_update_google_font_json( $api_key ) {

		$google = wp_remote_get( 'https://www.googleapis.com/webfonts/v1/webfonts?key=' . $api_key );

		if ( 200 !== wp_remote_retrieve_response_code( $google ) ) {
			return;
		}

		$google = wp_remote_retrieve_body( $google );
		$google = json_decode( $google, true );

		$new = array();
		foreach ( $google['items'] as $font ) {
			
			$new[ $font['family'] ]['weights']  = array();
			$new[ $font['family'] ]['styles'][] = 'normal';

			foreach ( $font['variants'] as $variant ) {

				if ( false !== strpos( $variant, 'italic' ) && ! in_array( 'italic', $new[ $font['family'] ]['styles'] ) ) {
					$new[ $font['family'] ]['styles'][] = 'italic';
				}

				$weight = str_replace( 'italic', '', $variant );
				$weight = str_replace( 'regular', 'normal', $weight );

				if ( $weight && ! in_array( $weight, $new[ $font['family'] ]['weights'] ) ) {
					$new[ $font['family'] ]['weights'][] = $weight;
				}
				
			}

			array_unique( $new[ $font['family'] ]['weights'] );
			array_unique( $new[ $font['family'] ]['styles'] );

			$new[ $font['family'] ]['subsets'] = $font['subsets'];
		}

		file_put_contents( HBTHEMES_ROOT . '/options-framework/data/gwf.json', json_encode( $new ) );
	}
endif;

if ( ! function_exists( 'highend_get_page_layout' ) ) :
	/**
	 * Get page sidebar layout.
	 *
	 * @since 3.5.1
	 */
	function highend_get_page_layout() {
		
		$layout = hb_options( 'hb_page_layout_sidebar' );

		if ( is_singular() ) {
			$singular_layout = vp_metabox( 'layout_settings.hb_page_layout_sidebar' ); 

			if ( 'default' !== $singular_layout && '' !== $singular_layout ) {
				$layout = $singular_layout;
			}
		}

		return apply_filters( 'highend_page_layout', $layout );
	}
endif;

if ( ! function_exists( 'highend_blog_class' ) ) :
	/**
	 * Classes for the main blog container div.
	 *
	 * @since 3.5.1
	 */
	function highend_blog_class( $template = '', $post_id = '' ) {

		$classes = array();

		$classes[] = 'clearfix';

		if ( 'blog' === $template ) {
			$classes[] = 'hb-blog-classic';
			$classes[] = 'hb-blog-large';
		} elseif ( 'blog-small' === $template ) {
			$classes[] = 'hb-blog-classic';
			$classes[] = 'hb-blog-large';
			$classes[] = 'hb-blog-small';
		} elseif ( 'blog-minimal' === $template ) {
		} elseif ( 'blog-grid' === $template ) {

			$classes[] = 'hb-blog-grid';
			$classes[] = 'masonry-holder';

			if ( $post_id ) {
				$classes[] = 'grid-columns-' . intval( vp_metabox( 'blog_grid_page_settings.hb_grid_columns', 3, $post_id ) );
			} elseif ( is_home() ) {
				$classes[] = 'grid-columns-' . intval( hb_options( 'hb_posts_page_grid_columns' ) );
			} elseif ( is_archive() ) {
				$classes[] = 'grid-columns-' . intval( hb_options( 'hb_archive_grid_columns' ) );
			} elseif ( is_search() ) {
				$classes[] = 'grid-columns-' . intval( hb_options( 'hb_search_grid_columns' ) );
			}

		} elseif ( 'blog-grid-fullwidth' === $template ) {

			$classes[] = 'hb-blog-grid';
			$classes[] = 'masonry-holder';

			if ( $post_id ) {
				$classes[] = 'grid-columns-' . intval( vp_metabox( 'blog_fw_page_settings.hb_grid_columns', 3, $post_id ) );
			} elseif ( is_home() ) {
				$classes[] = 'grid-columns-' . intval( hb_options( 'hb_posts_page_grid_columns' ) );
			} elseif ( is_archive() ) {
				$classes[] = 'grid-columns-' . intval( hb_options( 'hb_archive_grid_columns' ) );
			} elseif ( is_search() ) {
				$classes[] = 'grid-columns-' . intval( hb_options( 'hb_search_grid_columns' ) );
			}
		}

		$classes = apply_filters( 'highend_blog_class', $classes, $template );
		$classes = trim( implode( ' ', $classes ) );

		echo esc_attr( $classes );
	}
endif;

if ( ! function_exists( 'highend_blog_attributes' ) ) :
	/**
	 * Attributes for the main blog container div.
	 *
	 * @since 3.5.1
	 */
	function highend_blog_attributes( $template, $post_id = '' ) {

		$data = array();

		if ( 'blog-grid' === $template ) {

			if ( is_singular( $post_id ) ) {
				$data['data-layout-mode'] = vp_metabox( 'blog_grid_page_settings.hb_grid_style', 'masonry', $post_id );
			} else {
				$data['data-layout-mode'] = 'masonry';
			}

		} elseif ( 'blog-grid-fullwidth' === $template ) {

			if ( is_singular( $post_id ) ) {
				$data['data-layout-mode'] = vp_metabox( 'blog_fw_page_settings.hb_grid_style', 'masonry', $post_id );
			} else {
				$data['data-layout-mode'] = 'masonry';	
			}

		}

		$data = apply_filters( 'highend_blog_data', $data, $template, $post_id );
		
		if ( empty( $data ) ) {
			return;
		}

		$output = '';

		foreach ( $data as $key => $value ) {
			$output .= esc_html( $key ) . '="' . esc_attr( $value ) . '" ';
		}

		echo trim( $output );
	}
endif;

if ( ! function_exists( 'highend_main_content_style' ) ) :
	/**
	 * Main content div style tag.
	 *
	 * @since 3.5.1
	 */
	function highend_main_content_style( $post_id = '' ) {

		$styles = array();

		$content_bg_color = vp_metabox( 'background_settings.hb_content_background_color', '', $post_id );
		if ( ! empty( $content_bg_color ) ) {
			$styles['background-color'] = $content_bg_color; 
		}

		$styles = apply_filters( 'highed_main_content_style', $styles );
		$style  = '';

		if ( ! empty( $styles ) ) {
			foreach ( $styles as $key => $value ) {
				$style .= $key . ':' . $value . ';';
			}
		}
		
		$style = ! empty( $style ) ? ' style="' . $style . '"' : '';

		echo $style;
	}
endif;

if ( ! function_exists( 'highend_get_page_template' ) ) :
	/**
	 * Get page template slug/id.
	 *
	 * @since 3.5.1
	 */
	function highend_get_page_template( $post_id = '' ) {

		$template = 'blog'; //todo from highend options

		if ( ! empty( $post_id ) ) {
			
			$slug = get_page_template_slug( $post_id );

			if ( 'page-blog.php' === $slug ) {
				$template = vp_metabox( 'blog_page_settings.hb_blog_style', '', $post_id );
				$template = 'blog-small' === $template ? 'blog-small' : 'blog';
			} elseif ( 'page-blog-minimal.php' === $slug ) {
				$template = 'blog-minimal';
			} elseif ( 'page-blog-grid.php' === $slug ) {
				$template = 'blog-grid';
			} elseif ( 'page-blog-fullwidth.php' === $slug ) {
				$template = 'blog-grid-fullwidth';
			}

		} elseif ( is_search() ) {
			$template = hb_options( 'hb_search_template' );
		} elseif ( is_archive() ) {
			$template = hb_options( 'hb_archive_template' );
		} elseif ( is_home() ) {
			$template = hb_options( 'hb_posts_page_template' );
		}

		return $template;
	}
endif;

if ( ! function_exists( 'highend_get_blog_query_args' ) ) :
	/**
	 * Blog page query args.
	 *
	 * @since 3.5.1
	 */
	function highend_get_blog_query_args( $post_id = '' ) {

		if ( get_query_var('paged') ) {
		    $paged = get_query_var('paged');
		} elseif ( get_query_var('page') ) {
		    $paged = get_query_var('page');
		} else {
		    $paged = 1;
		}

		$template = highend_get_page_template( $post_id );

		$orderby = $order = $category__in = '';

		if ( 'blog' === $template || 'blog-small' === $template ) {
			$orderby      = vp_metabox( 'blog_page_settings.hb_query_orderby', 'date', $post_id );
			$order        = vp_metabox( 'blog_page_settings.hb_query_order', 'DESC', $post_id );
			$category__in = vp_metabox( 'blog_page_settings.hb_blog_category_include', '', $post_id );
		} elseif ( 'blog-minimal' === $template ) {
			$orderby      = vp_metabox( 'blog_page_minimal_settings.hb_query_orderby', 'date', $post_id );
			$order        = vp_metabox( 'blog_page_minimal_settings.hb_query_order', 'DESC', $post_id );
			$category__in = vp_metabox( 'blog_page_minimal_settings.hb_blog_category_include', '', $post_id );
		} elseif ( 'blog-grid' === $template ) {
			$orderby      = vp_metabox( 'blog_grid_page_settings.hb_query_orderby', 'date', $post_id );
			$order        = vp_metabox( 'blog_grid_page_settings.hb_query_order', 'DESC', $post_id );
			$category__in = vp_metabox( 'blog_grid_page_settings.hb_blog_category_include', '', $post_id );
		} elseif ( 'blog-grid-fullwidth' === $template ) {
			$orderby      = vp_metabox( 'blog_fw_page_settings.hb_query_orderby', 'date', $post_id );
			$order        = vp_metabox( 'blog_fw_page_settings.hb_query_orderby.hb_query_order', 'DESC', $post_id );
			$category__in = vp_metabox( 'blog_fw_page_settings.hb_blog_category_include', '', $post_id );
		}

		return array(
			'post_type'      => 'post',
			'paged'          => $paged,
			'posts_per_page' => get_option('posts_per_page'),
			'orderby'        => $orderby,
			'order'          => $order,
			'category__in'   => $category__in,
			'post_status'    => 'publish',
		);
	}
endif;
